﻿//////////////////////////////////////////////
// Compositor.h
//
//////////////////////////////////////////////

/// Defines / macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class CompositorNode ;
	class Texture ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Resources/Resource.h"

// Export
#include <NilkinsExport/Exportable/Exportable.h>

// nkMemory
#include <NilkinsMemory/Containers/BufferCast.h>
#include <NilkinsMemory/Containers/String.h>

#include <NilkinsMemory/Pointers/UniquePtr.h>

/// Classe -----------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Compositor : public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Nodes making our rendering chain
			nkMemory::BufferCast<nkMemory::UniquePtr<CompositorNode>> _renderChain ;
			
			// Info on compositor
			nkMemory::String _resourceName ;

			// Flags
			bool _hidden ;
		
		public :
		
			// Constructeur, destructeur
			Compositor () ;
			Compositor (const Compositor&) = delete ;
			virtual ~Compositor () ;

			// Manipulations nodes
			virtual CompositorNode* addNode () = 0 ;
			virtual void deleteNode (unsigned int index) ;
			virtual void changeNodeOrder (unsigned int fromIndex, unsigned int toIndex) ;
			virtual void clearNodes () ;

			// Getters
			unsigned int getNodeCount () const ;
			CompositorNode* getNode (unsigned int index) const ;
			nkMemory::StringView getResourceName () const ;
			bool getHidden () const ;

			// Setters
			void setResourceName (const nkMemory::StringView& value) ;
			void setHidden (bool value) ;

			// Rendering
			virtual void renderChain () ;

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;

			// Operators
			Compositor& operator= (const Compositor&) = delete ;
	} ;
}