﻿//////////////////////////////////////////////
// CompositorManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Compositor ;
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

#include <NilkinsMemory/Pointers/UniquePtr.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT CompositorManager final : public nkCommon::SingletonClass<CompositorManager>
	{
		friend class nkCommon::SingletonClass<CompositorManager> ;

		private :
		
			// Attributs
			// Une map des Compositor enregistrés
			std::unordered_map<std::string, nkMemory::UniquePtr<Compositor>> _memory ;

			// Threading
			std::mutex _mutex ;

			// Le compositor actif
			Compositor* _activeCompositor ;
			// Le Compositor par défaut
			Compositor* _defaultCompositor ;
		
		public :
		
			// Constructeur, destructeur
			CompositorManager () ;
			~CompositorManager () ;

			// Getters
			Compositor* getActiveCompositor () const ;
			Compositor* getDefaultCompositor () const ;
			
			// Enregistrement
			Compositor* createOrRetrieve (const nkMemory::StringView& name) ;
			Compositor* get (const nkMemory::StringView& name) ;
			Compositor* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Setters
			void setActiveCompositor (Compositor* compositor) ;
			void setDefaultCompositor (Compositor* compositor) ;

			// Activations lors des changements de contextes
			void prepareForContext (RenderContext* context) ;
	} ;
}