﻿//////////////////////////////////////////////
// ProgramManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Program ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Utils/InfiniteMask.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <map>
#include <unordered_map>
#include <mutex>
#include <string>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramManager final : public nkCommon::SingletonClass<ProgramManager>
	{
		friend class nkCommon::SingletonClass<ProgramManager> ;

		private :
		
			// Attributs
			// Mutex unique pour tout, à voir si bénéfice à split
			std::mutex _mutex ;

			// Mémoire des shaders instanciés
			std::unordered_map<std::string, Program*> _programMap ;

			// Compteurs pour les ids
			unsigned int _programCounter ;

		private :

			// Functions
			// Constructor
			ProgramManager () ;
		
		public :

			// Destructor
			~ProgramManager () ;

			// Manipulating memory
			Program* createOrRetrieve (const nkMemory::StringView& name) ;
			Program* get (const nkMemory::StringView& name) ;
			Program* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;
	} ;
}