﻿//////////////////////////////////////////////
// ThemeManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward Decl -----------------------------

namespace nkWinUi
{
	class MainSystem ;
	class Theme ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/MaybeSingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <memory>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT ThemeManager final : public nkCommon::MaybeSingletonClass<ThemeManager>
	{
		friend class nkCommon::MaybeSingletonClass<ThemeManager> ;
		friend class MainSystem ;

		private :

			// Memory
			std::unordered_map<std::string, Theme*> _themes ;

			MainSystem* _parentSystem ;
			Theme* _activeTheme ;

		private :
	
			// Functions
			// Constructor, destructor
			ThemeManager () ;
			ThemeManager (MainSystem* parentSystem) ;
			~ThemeManager () ;

		public :

			// Manipulations
			Theme* createOrRetrieve (const nkMemory::StringView& name) ;
			Theme* get (const nkMemory::StringView& name) const ;

			// Getters
			Theme* getActiveTheme () const ;

			// Setters
			void setActiveTheme (Theme* theme) ;
	} ;
}